;
; Oink! ZX Spectrum 128 Patch
;
; Patch Oink!, which was a tape multi-load for the 48k ZX Spectrum,
; to use the Spectrum 128's extra memory so that the three sub-games
; are all loaded into RAM at once.
;
; Mark Incley, 16th February 2014.
;
; Assemble with SjASMPlus.
;

		DEVICE ZXSPECTRUM48
; The patch lives immediately after the IM2 vector table which is located
; between 0xB600->0xB700 (and points to 0xB5B5)
		ORG	0xB701

; 0x4000 - 0x7FFF Bank 5
; 0x8000 - 0xBFFF Bank 2
; 0xC000 - 0xFFFF Bank 0 and 1, 3, 4 or 6 (see below)
;
; The sub-games have been designated thus:
; Bank 1 - Pete's Pimple
; Bank 3 - Rubbishman (part 1)
; Bank 4 - Tom Thug
; Bank 6 - Rubbishman (part 2)

LoadOinkFile	EQU	0x88EA
FileTable	EQU	0x8BEF	; word start, length
InitGame	EQU	0x80A2
ClearAttr	EQU	0x8AFE

FilePetesPimple	EQU	0x81
FileRubbishP1	EQU	0x84
FileRubbishP2	EQU	0x82
FileTomThug	EQU	0x83

ProgramStart:
;
; The call to InitGame at 0x8052 should be patched to call PatchedInitGame (0xB701) by the .POK file.
;
PatchedInitGame:
		ld	hl,havePatched
		ld	a,(hl)
		or	a
		jr	nz,.patched

		inc	(hl)

		; Before we patch the file loader, use it to load all files to RAM
		call	LoadAllFiles

		ld	hl,LoadOinkFile
		ld	(hl),195		; jp LoadFileFromRAM
		inc	hl
		ld	(hl),LOW LoadFileFromRAM
		inc	hl
		ld	(hl),HIGH LoadFileFromRAM

.patched:	jp	InitGame		; jump back to the original Oink code to initialise the game


;  In: (currentFile) = file number 80-84h
; Out: hl = address
;      bc = length
GetFileEntry:	ld	a,(currentFile)
		add	a,a
		add	a,a
		ld	l,a
		ld	h,0
		ld	de,FileTable
		add	hl,de
		ld	e,(hl)
		inc	hl
		ld	d,(hl)
		inc	hl
		ld	c,(hl)
		inc	hl
		ld	b,(hl)		; bc = length
		ex	de,hl
		ret


LoadAllFiles:
		; Patch the file loader so that it always loads to address 0xC000
		; regardless of the values in the file's header
		ld	hl,0x8985
		inc	(hl)		; Ensure HL is moved on, as we're going
		inc	hl		; to clobber its "inc hl" with the following
		inc	hl		; code change:
		ld	(hl),1		; ld bc,0xc000
		inc	hl
		ld	(hl),0
		inc	hl
		ld	(hl),0xc0

		ld	a,FilePetesPimple
		call	LoadFile

		ld	a,FileRubbishP1
		call	LoadFile

		ld	a,FileRubbishP2
		call	LoadFile

		ld	a,FileTomThug
		call	LoadFile

		;
		; Fall through to PageRAMZero
		;


PageRAMZero:	xor	a

		;
		; Fall through to PageRAM
		;

; PageRAM
;	 In: al = RAM bank
;	Out: Memory bank paged in

PageRAM:	or	00010000b
		ld	bc,7ffdh
		out	(c),a
		ret


LoadFile:	ld	(currentFile),a
		call	SelectFileRAM

.retry:		ld	a,(currentFile)
		call	LoadOinkFile	; Load the file directly into the correct RAM page
		jr	nc,.retry	; Any error try to re-load. NB carry = success
		ret


SelectFileRAM:	and	7
		ld	l,a
		ld	h,0
		ld	de,RAMPages
		add	hl,de
		ld	a,(hl)		; a = source RAM page
		jr	PageRAM


LoadFileFromRAM:
		di

; We need to handle Rubbishman differently, as it is loaded in two parts:
; Part 1 (file # 84h) is loaded to 0xb801 then moved down to 0x5b00
; Part 2 (file # 82h) is loaded to 0xb801
; Because we're using the 0x5b00 area (as well as the screen) as a temporary buffer
; to hold data from paged RAM, we can't use that approach any more.
; Instead, we fake loading of file 84h, which will result in garbage being copied to 0x5b00
; by the original code, but then we can handle both parts of the scenario when file 82h
; is loaded afterwards. Sneaky huh?

		cp	FileRubbishP1	; Pretend rubbishman part 1 loaded OK
		jr	z,.out

		cp 	FileRubbishP2	; Rubbishman part 2?
		jr	nz,.load	; No, it's Pete's Pimple or Tom Thug

		call	.load		; Load file #82 (part 2) directly into 0xb801

		ld	a,FileRubbishP1	; Get the entry for Rubbishman part 1
		ld	(currentFile),a
		call	SelectFileRAM
		call	GetFileEntry
		ld	hl,0xc000	; Copy it down to 0x5b00
		ld	de,0x5b00
		ldir
		call	PageRAMZero	; Restore memory page and we're done
		jr	.out

.load:		ld	(currentFile),a
		call	SelectFileRAM
		call	GetFileEntry
		push	hl
		ld	hl,0xc000
		ld	de,0x4000
		push	bc
		push	de

		ldir		; Copy game to screen

		call	PageRAMZero	; Select "normal" page for 0xc000

		pop	hl
		pop	bc
		pop	de
		ldir		; Copy from screen to page 0

.out:		scf		; Return with CY = success
		ret


havePatched	db	0
currentFile	db	0
RAMPages	db	0, 	1, 	3, 	4, 	6


		SAVEBIN "oink128.bin",ProgramStart,$-ProgramStart

		DISPLAY "Code End: ", $
